"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Either = require("fp-ts/lib/Either");
const pg = require("./pg");
const schema_1 = require("./schema");
const tstype_1 = require("./tstype");
function connect(connectionString) {
    return __awaiter(this, void 0, void 0, function* () {
        const pgClient = new pg.Client(connectionString == null ? undefined : { connectionString });
        try {
            yield pgClient.connect();
        }
        catch (err) {
            return Either.left(`Error connecting to database: ${err.message}`);
        }
        const schema = schema_1.schemaClient(pgClient);
        const types = yield tstype_1.typeClient(pgClient);
        return Either.right({ pg: pgClient, schema, types });
    });
}
exports.connect = connect;
function disconnect(clients) {
    return __awaiter(this, void 0, void 0, function* () {
        yield clients.pg.end();
    });
}
exports.disconnect = disconnect;
function clearCache(clients) {
    return __awaiter(this, void 0, void 0, function* () {
        // The type client caches stuff about user-defined SQL types.
        // Recreate it to clear the cache.
        const types = yield tstype_1.typeClient(clients.pg);
        return Object.assign(Object.assign({}, clients), { types });
    });
}
exports.clearCache = clearCache;
