"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Option = require("fp-ts/lib/Option");
const pipeable_1 = require("fp-ts/lib/pipeable");
const constants_1 = require("./constants");
function findOperator(name) {
    const upCaseName = name.toUpperCase();
    return Option.fromNullable(constants_1.operators.find(op => op.op === upCaseName));
}
exports.findOperator = findOperator;
function isOperatorCommutative(name) {
    return pipeable_1.pipe(findOperator(name), Option.map(op => op.commutative || false), Option.getOrElse(() => false));
}
exports.isOperatorCommutative = isOperatorCommutative;
function operatorNullSafety(name) {
    return pipeable_1.pipe(findOperator(name), Option.map(op => op.nullSafety), Option.toNullable);
}
exports.operatorNullSafety = operatorNullSafety;
function findFunction(name) {
    const downCaseName = name.toLowerCase();
    return Option.fromNullable(constants_1.functions.find(f => f.name === downCaseName));
}
exports.findFunction = findFunction;
function functionNullSafety(name) {
    return pipeable_1.pipe(findFunction(name), Option.map(func => func.nullSafety), Option.toNullable);
}
exports.functionNullSafety = functionNullSafety;
