"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const R = require("ramda");
const Either_1 = require("fp-ts/lib/Either");
function preprocessSQL(sql) {
    const namedParamMatch = sql.match(/\$\{\w+\}/g);
    const numberedParamMatch = sql.match(/\$\d+/g);
    if (namedParamMatch != null && numberedParamMatch != null) {
        const firstNamedParam = namedParamMatch[0];
        const firstNumberedParam = numberedParamMatch[0];
        return Either_1.left(`Cannot mix named parameters (${firstNamedParam}) and numbered \
parameters (${firstNumberedParam}) in the same statement`);
    }
    else if (namedParamMatch == null && numberedParamMatch == null) {
        return Either_1.right({ sql, paramNames: [] });
    }
    else if (namedParamMatch != null) {
        // Only named params
        return Either_1.right(handleNamedParams(sql, namedParamMatch));
    }
    else if (numberedParamMatch != null) {
        // Only numbered params
        return Either_1.right(handleNumberedParams(sql, numberedParamMatch));
    }
    else {
        throw new Error('not reached');
    }
}
exports.preprocessSQL = preprocessSQL;
function handleNamedParams(sql, params) {
    const paramIndices = new Map();
    let current = 1;
    params.forEach(param => {
        const name = param.slice(2, -1);
        if (!paramIndices.has(name)) {
            paramIndices.set(name, current);
            current++;
        }
    });
    let mangledSQL = sql;
    for (const [name, index] of paramIndices) {
        mangledSQL = mangledSQL.replace(new RegExp('\\$\\{' + name + '\\}', 'g'), '$' + index);
    }
    // Iterating a Map is guaranteed to yield in the insertion order
    return { sql: mangledSQL, paramNames: [...paramIndices.keys()] };
}
function handleNumberedParams(sql, numberedParams) {
    return {
        sql,
        paramNames: R.sort((a, b) => a.localeCompare(b), numberedParams),
    };
}
